uniform float alpha;
uniform sampler2D Texture;
uniform sampler2DShadow ShadowMap;
uniform int TexOn;
uniform int ShadowOn;

float PCFlookup()
{
	// generate the 9 texture co-ordinates for a 3x3 PCF kernel
	vec4 vTexCoords[9];
	// texel size
	//float fTexelSize = 1.0 / 2048.0;
	float fTexelSize = 0.05; // values between 0.01 and 0.1 seem ok... 

	// generate the tecture co-ordinates for the specified depth-map size
	// 4 3 5
	// 1 0 2
	// 7 6 8
	vTexCoords[0] = gl_TexCoord[1];
	vTexCoords[1] = gl_TexCoord[1] + vec4( -fTexelSize, 0.0, 0.0, 0.0 );
	vTexCoords[2] = gl_TexCoord[1] + vec4(  fTexelSize, 0.0, 0.0, 0.0 );
	vTexCoords[3] = gl_TexCoord[1] + vec4( 0.0, -fTexelSize, 0.0, 0.0 );
	vTexCoords[6] = gl_TexCoord[1] + vec4( 0.0,  fTexelSize, 0.0, 0.0 );
	vTexCoords[4] = gl_TexCoord[1] + vec4( -fTexelSize, -fTexelSize, 0.0, 0.0 );
	vTexCoords[5] = gl_TexCoord[1] + vec4(  fTexelSize, -fTexelSize, 0.0, 0.0 );
	vTexCoords[7] = gl_TexCoord[1] + vec4( -fTexelSize,  fTexelSize, 0.0, 0.0 );
	vTexCoords[8] = gl_TexCoord[1] + vec4(  fTexelSize,  fTexelSize, 0.0, 0.0 );
	// sample each of them checking whether the pixel under test is shadowed or not
	float fShadowTerms[9];
	float fShadowTerm = 0.0;

	// clamp the shadowmap projection to forward only
	float z = gl_TexCoord[1].z / gl_TexCoord[1].w;
	float c = step(0.6, z) * step(z, 1.0);

	for( int i = 0; i < 9; i++ )
	{
		float depth = shadow2DProj(ShadowMap, vTexCoords[i]).r * c;

		// Texel is shadowed
		fShadowTerms[i] = depth != 1.0 ? 0.75 : 1.0;
		fShadowTerm += fShadowTerms[i];
	}
	
	// get the average
	fShadowTerm /= 9.0;

	return fShadowTerm;
}

void main()
{
	float shadeFactor = 1.0;
	
	if (ShadowOn == 1)
	{
		shadeFactor = PCFlookup();
	}
		
	vec4 texels = texture2D(Texture, vec2(gl_TexCoord[0]));
    
	if (TexOn == 1)
		gl_FragColor = vec4(shadeFactor * gl_Color.rgb, alpha) * texels;
	else
		gl_FragColor = vec4(shadeFactor * gl_Color.rgb, alpha);
}