uniform mat4 shadowMatrix;

void PointLight(in int i,
                in vec3 eye,
                in vec3 ecPosition3,
                in vec3 normal,
                inout vec4 ambient,
                inout vec4 diffuse,
                inout vec4 specular)
{
    float nDotL;          // normal . light direction
    float nDotH;          // normal . light half vector
    float pf;             // power factor
    float attenuation;    // computed attenuation factor
    float d;              // distance from surface to light source
    vec3  L;              // direction from surface to light position
    vec3  halfVector;     // direction of maximum highlights

    // Compute vector from surface to light position
    L = vec3(gl_LightSource[i].position) - ecPosition3;

    // Compute distance between surface and light position
    d = length(L);

    // Normalize the vector from surface to light position
    L = normalize(L);

    // Compute attenuation
    attenuation = 1.0 / (gl_LightSource[i].constantAttenuation +
                         gl_LightSource[i].linearAttenuation * d +
                         gl_LightSource[i].quadraticAttenuation * d * d);

    halfVector = normalize(L + eye); 

    nDotL = max(0.0, dot(normal, L));
    nDotH = max(0.0, dot(normal, halfVector));

    if (nDotH == 0.0)
        pf = 0.0;
    else
        pf = pow(nDotH, gl_FrontMaterial.shininess);

    ambient += gl_LightSource[i].ambient * attenuation;
    diffuse += gl_LightSource[i].diffuse * nDotL * attenuation;
    specular += gl_LightSource[i].specular * pf * attenuation;
}

void main(void)
{
	vec4 amb  = vec4(0.0);
	vec4 diff = vec4(0.0);
	vec4 spec = vec4(0.0);

	vec4 ecPosition  = gl_ModelViewMatrix * gl_Vertex;
    vec3 ecPosition3 = (vec3(ecPosition)) / ecPosition.w;

	gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
	vec3 normal = normalize(gl_NormalMatrix * gl_Normal);
	
	vec3 eye = normalize(-ecPosition3);

	PointLight(0, eye, ecPosition3, normal, amb, diff, spec);
	
	vec4 color = gl_FrontLightModelProduct.sceneColor +
            amb * gl_FrontMaterial.ambient +
            diff * gl_FrontMaterial.diffuse;
            
    color += spec * gl_FrontMaterial.specular * 4.0; // additional multiplier to make it look better
	gl_FrontColor = gl_Color * color;

	gl_TexCoord[0] = gl_MultiTexCoord0;
	gl_TexCoord[1] = shadowMatrix * ecPosition;
}